/***************************************************************************
 *                                                                         *
 *   KNetLoad is copyright (c) 1999-2000, Markus Gustavsson                *
 *                         (c) 2002, Ben Burton                            *
 *                         (c) 2004-2005, Diego Pettenò                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "netloadapplet.h"
#include "../version.h"
#include "settings.h"
#include "interfaces.h"
#include "visualization.h"
#include "knetproc.h"
#include "statgraph.h"
#include "statpopup.h"
#include "interfaceslist.h"

#include <kaboutapplication.h>
#include <kconfigdialog.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <kcombobox.h>
#include <kpushbutton.h>
#include <klistview.h>
#include <qcheckbox.h>
#include <qtimer.h>
#include <qlabel.h>
#include <qregexp.h>

NetLoadApplet::NetLoadApplet(const QString& configFile, Type type, int actions, QWidget *parent, const char *name)
 : KPanelApplet(configFile, type, actions, parent, name)
{
	kdDebug() << "NetLoadApplet::NetLoadApplet(" << configFile << ", " << type << ", " << actions << ", " << parent << ", " << name << ")" << endl;
	KAboutData aboutData(
		"knetloadapplet",
		I18N_NOOP("KNetLoad Applet"),
		VERSION,
		I18N_NOOP("A Kicker applet of KNetLoad"),
		KAboutData::License_GPL,
		"(c) 1999-2000, Markus Gustavsson\n"
		"(c) 2002, Ben Burton\n"
		"(c) 2004-2005, Diego Pettenò",
		I18N_NOOP("Code took from old KNetLoad standalone application"),
		"http://flameeyes.web.ctonet.it/");

	aboutData.addAuthor("Markus Gustavsson", I18N_NOOP("Original author"), "mighty@fragzone.se");
	aboutData.addAuthor("Ben Burton", I18N_NOOP("KDE3 rewrite"), "bab@debian.org");
	aboutData.addAuthor("Diego Pettenò", I18N_NOOP("2.x+ Versions, current maintainer"), "flameeyes@users.berlios.de");
	aboutData.addAuthor("Roland Riegel", I18N_NOOP("BSD and Solaris support"), "feedback@roland-riegel.de");
	aboutData.addAuthor("Bill Wilson", I18N_NOOP("BSD and Solaris support"), "bill@gkrellm.net");
	aboutData.addAuthor("Hajimu Umemoto", I18N_NOOP("BSD support"), "ume@mahoroba.org");
	aboutData.addAuthor("Anthony Mallet", I18N_NOOP("BSD support"), "anthony.mallet@useless-ficus.net");
	aboutData.addAuthor("Daisuke Yabuki", I18N_NOOP("Solaris support"), "dxy@acm.org");
	
	aboutData.addCredit("Njaard, Charles Samuels", I18N_NOOP("Support and help with KDE"), "charles@kde.org");
	aboutData.addCredit("rikkus, Rik Hemsley", I18N_NOOP("Support and help with KDE"), "rik@kde.org");
	aboutData.addCredit("Mark Halpaap", I18N_NOOP("Independent port to KDE3"), "mark.halpaap@gmx.net");
	aboutData.addCredit(I18N_NOOP("Thanks to everyone in #KDE OpenProjects for being supportive and helpful!"));
	aboutData.addCredit(I18N_NOOP("Thanks obviously to all the creators of, maintainers of, and contributors to the original KNetLoad Application."));
	m_aboutbox = new KAboutApplication(&aboutData, this, "aboutbox");
	
	m_sets = new Settings("");
	m_sets->readConfig();

	m_readTimer = new QTimer(this);
	m_cfgdialog = new KConfigDialog(this, "settings", m_sets);
	
	VisualizationConfig *viscfg = new VisualizationConfig(m_cfgdialog, "Visualization");
	m_cfgdialog->addPage(viscfg, i18n("Visualization"), "style");
	
	InterfacesList *iflist = new InterfacesList(m_cfgdialog, "InterfacesList");
	connect(iflist->btnAdd, SIGNAL(clicked()), this, SLOT(interfaceAdd()));
	connect(iflist->btnRemove, SIGNAL(clicked()), this, SLOT(interfaceRemove()));
	connect(iflist->btnEdit, SIGNAL(clicked()), this, SLOT(interfaceEdit()));
	m_cfgdialog->addPage(iflist, i18n("Interfaces"), "network");
	m_interfaces = iflist->listIFs;
	
	connect(m_cfgdialog, SIGNAL(settingsChanged()), this, SLOT(reloadSettings()));

	m_readTimer->start(m_sets->timeout());

	reloadDevices();
}

NetLoadApplet::~NetLoadApplet()
{
	delete m_sets;
	delete m_aboutbox;
}

void NetLoadApplet::reloadDevices()
{
	{
		const QPtrList<KNetProc>::const_iterator end = m_procs.end();
		for(QPtrList<KNetProc>::iterator it = m_procs.begin(); it != end; it++)
		{
			delete *it;
			m_procs.erase(it);
		}
		m_procGroups.clear();
	}
	
	static QRegExp re("interface_(\\d+)");
	
	m_interfaces->clear();
	
	QStringList groups = m_sets->config()->groupList();
	const QStringList::const_iterator end = groups.end();
	for(QStringList::iterator it = groups.begin(); it != groups.end(); it++)
	{
		if ( re.search(*it) == -1 )
			continue;
		
		KNetProc *proc = new KNetProc(this, re.cap(1).toUInt());
		connect(proc->graphIn(), SIGNAL(clickedRight()), this, SLOT(preferences()));
		connect(proc->graphOut(), SIGNAL(clickedRight()), this, SLOT(preferences()));
		connect(m_readTimer, SIGNAL(timeout()), proc, SLOT(takeReading()));
		m_procs.append(proc);
	
		Settings ifsets(re.cap(1));
		ifsets.readConfig();
		QString title;
		
		if ( ifsets.type() == Settings::EnumType::Local )
		{
			title = "Local " + ifsets.name();
		} else {
			title = "Remote " + ifsets.name() + " @ " + ifsets.server();
		}
		
		m_interfaces->insertItem(title);
		m_procGroups.append(re.cap(1));
	}
	
	resizeEvent(NULL);
}

int NetLoadApplet::widthForHeight(int height) const
{
	return m_procs.count() * ( height*2 + 4 );
}

int NetLoadApplet::heightForWidth(int width) const
{
	return (width/m_procs.count()-4)/2;
}

void NetLoadApplet::about()
{
	kdDebug() << "NetLoadAPplet::about()" << endl;
	m_aboutbox->show();
}

void NetLoadApplet::preferences()
{
	kdDebug() << "NetLoadApplet::preferences()" << endl;
	
	if(KConfigDialog::showDialog("settings"))
		return;
	
	m_cfgdialog->show();
}

void NetLoadApplet::reloadSettings()
{
	kdDebug() << "NetLoadApplet::reloadSettings()" << endl;
	m_sets->writeConfig();
	
	const QPtrList<KNetProc>::const_iterator end = m_procs.end();
	for(QPtrList<KNetProc>::iterator it = m_procs.begin(); it != end; it++)
		(*it)->reloadSettings();
}

void NetLoadApplet::resizeEvent(QResizeEvent *e)
{
	kdDebug() << "NetLoadApplet::resizeEvent(" << e << ")" << endl;
	KPanelApplet::resizeEvent(e);
	
	uint i = 0;
	const QPtrList<KNetProc>::const_iterator end = m_procs.end();
	for(QPtrList<KNetProc>::iterator it = m_procs.begin(); it != end; it++)
	{
		const uint pos = i*(2*height()+4);
		(*it)->graphIn()->setGeometry(pos, 0, height(), height());
		(*it)->graphIn()->show();
			
		(*it)->graphOut()->setGeometry(pos+height()+2, 0, height(), height());
		(*it)->graphOut()->show();
		
		i++;
	}
	
	update();
	reloadSettings();
}

void NetLoadApplet::interfaceAdd()
{
	// Here we limit to 256 interfaces, this can change.. but who needs to
	// monitor more than 256 interfaces in a kicker applet? o_O
	uint i;
	for(i = 0; i < 256; i++)
		if( ! m_sets->config()->hasGroup("interface_" + QString::number(i)) )
			break;

	configureIf(QString::number(i));
}

void NetLoadApplet::interfaceRemove()
{
	kdDebug() << "NetLoadApplet::interfaceRemove()" << endl;
	int itm = m_interfaces->currentItem();
	if ( itm == -1 ) return;

	delete m_procs.at(itm);
	m_procs.remove(itm);
	
	m_sets->config()->deleteGroup("interface_" + *(m_procGroups.at(itm)));
	m_sets->writeConfig();

	reloadDevices();
}

void NetLoadApplet::interfaceEdit()
{
	int itm = m_interfaces->currentItem();
	if ( itm == -1 ) return;
	configureIf(*(m_procGroups.at(itm)));
}

void NetLoadApplet::configureIf(const QString &param)
{
	Settings ifsets(param);
	
	ifsets.readConfig();
	KConfigDialog *ifadd = new KConfigDialog(this, "settings", &ifsets, KDialogBase::Plain);
	InterfacesConfig *ifcfg = new InterfacesConfig(ifadd->plainPage(), "Interfaces");
	const QStringList &ifaces = KNetProc::loadInterfaces(true);
	QStringList::const_iterator enddev = ifaces.end();
	for(QStringList::const_iterator it = ifaces.begin(); it != enddev; it++)
		ifcfg->kcfg_Name->insertItem(KNetProc::getInterfaceIcon(*it), *it);

	ifadd->addPage(ifcfg, i18n("Interface"), "network");
	
	if ( ifadd->exec() == QDialog::Accepted )
	{
		ifsets.writeConfig();
		m_sets->readConfig();
	}
	
	reloadDevices();
	
	delete ifadd;
}

extern "C"
{
	KPanelApplet* init( QWidget *parent, const QString configFile)
	{
		kdDebug() << "[NetLoadApplet] init(" << parent << ", " << configFile << ")" << endl;
		KGlobal::locale()->insertCatalogue("knetloadapplet");
		return new NetLoadApplet(configFile, KPanelApplet::Normal,
			KPanelApplet::About | KPanelApplet::Preferences,
			parent, "knetloadapplet");
	}
}

#include "netloadapplet.moc"
